<?php

	/*
	* Plugin Name: Add Meta to Shipping Labels
	* Plugin URI: https://www.pluginhive.com/plugins/
	* Description: This plugin will use Order / Order Item / Product Meta Data Value to display in Shipping Label.
	* Author: PluginHive
	* Author URI: https://pluginhive.com/
	* Version: 1.0.3
	* Text Domain: ph_meta_data_to_shipping_labels
	* Copyright: 2020 PluginHive.
	* WC requires at least: 3.0.0
	* WC tested up to: 5.3.0
	*/

	if ( ! defined( 'ABSPATH' ) ) {
		exit;
	}

	function ph_check_for_meta_to_label_required_plugins() {

		$ph_addon_required_plugins 	= false;

		$active_plugins = apply_filters( 'active_plugins', get_option( 'active_plugins', array() ) );

		// Multisite case
		if ( is_multisite() ) {
			$active_plugins = array_merge( $active_plugins, array_keys( get_site_option( 'active_sitewide_plugins', array() ) ) );
		}

		if ( in_array( 'fedex-woocommerce-shipping/fedex-woocommerce-shipping.php', $active_plugins ) || in_array( 'ups-woocommerce-shipping/ups-woocommerce-shipping.php', $active_plugins ) ) {

			$ph_addon_required_plugins = true;
		}

		return $ph_addon_required_plugins;
	}

	// Auto Deactivate
	add_action( 'admin_init', 'ph_meta_to_customer_ref_addon_deactivate' );

	if ( ! function_exists( 'ph_meta_to_customer_ref_addon_deactivate' ) ) {
		function ph_meta_to_customer_ref_addon_deactivate() {

			$ph_required_plugins 	= ph_check_for_meta_to_label_required_plugins();

			if ( !$ph_required_plugins ) {

				deactivate_plugins( plugin_basename( __FILE__ ) );
				wp_safe_redirect( admin_url('plugins.php') );
			}
		}
	}

	function ph_meta_to_customer_ref_activation_check(){

		$ph_required_plugins 	= ph_check_for_meta_to_label_required_plugins();

		if ( !$ph_required_plugins ) {

			deactivate_plugins( basename( __FILE__ ) );
			wp_die( __("Oops! You tried installing the Add Meta to Shipping Labels without activating WooCommerce FedEx Shipping Plugin with Print Label / WooCommerce UPS Shipping Plugin with Print Label. Kindly activate the dependent plugin and then try again", "ph_meta_data_to_shipping_labels" ), "", array('back_link' => 1 ));
		}
	}

	register_activation_hook( __FILE__, 'ph_meta_to_customer_ref_activation_check' );

	function PH_meta_data_to_shipping_labels() {

		if( !class_exists('PH_Meta_Data_To_Shipping_Labels') ){
			class PH_Meta_Data_To_Shipping_Labels extends WC_Shipping_Method{

				public function __construct() {

					$this->id		= 'ph_meta_data_to_shipping_labels';

					$this->method_title       = __( 'Add Meta Data to Shipping Labels', 'ph_meta_data_to_shipping_labels');
					$this->method_description = __( 'This plugin will use Order / Order Item / Product Meta Data Value to display in Shipping Label.', 'ph_meta_data_to_shipping_labels');

					$this->init();
				}

				function init() {

					$this->init_form_fields();
					$this->init_settings();

					add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );
				}

				function init_form_fields() {

					$this->form_fields = array(
                        'ph_data_type' => array(
							'title'			=> __('Data Type', 'ph_meta_data_to_shipping_labels'),
							'type'			=> 'select',
							'default'		=> 'metaData',
							'options'		=> array(
								'metaData' 	=> __('Meta data', 'ph_meta_data_to_shipping_labels'),
								'customData' 	=> __('Custom Data', 'ph_meta_data_to_shipping_labels'),
							),
							'description'	=> __('To select type of data to be passed.','ph_meta_data_to_shipping_labels'),
				            'desc_tip'		=> true,
						),
						'ph_custom_field' => array(
							'title'			=> __('Custom Data', 'ph_meta_data_to_shipping_labels'),
							'type'			=> 'textarea',
							'description'	=> __('Supported Tags:[ORDER_ID]: Order Id','ph_meta_data_to_shipping_labels'),
							'placeholder'	=>	__('[ORDER_ID]', 'ph_meta_data_to_shipping_labels'),
						),

						'ph_meta_data_type' => array(
							'title'			=> __('Meta Data Type', 'ph_meta_data_to_shipping_labels'),
							'type'			=> 'select',
							'options'		=> array(
								'order_meta' 		=> __('Order Meta', 'ph_meta_data_to_shipping_labels'),
								'order_item_meta' 	=> __('Order Item Meta', 'ph_meta_data_to_shipping_labels'),
								'product_meta' 		=> __('Product Meta', 'ph_meta_data_to_shipping_labels'),
							),
							'description'	=> __('','ph_meta_data_to_shipping_labels'),
						),

						'ph_meta_data_key' => array(
							'title'			=> __('Meta Data Value', 'ph_meta_data_to_shipping_labels'),
							'type'			=> 'text',
							'description'	=> __('','ph_meta_data_to_shipping_labels'),
						),

						'ph_fedex_label_enable' => array(
							'title'			=> __('Enable Meta for FedEx', 'ph_meta_data_to_shipping_labels'),
							'type'			=> 'checkbox',
							'default' 		=> 'no',
							'label'			=> __('Enable','ph_meta_data_to_shipping_labels'),
							'description'	=> __('','ph_meta_data_to_shipping_labels'),
						),

						'ph_fedex_label_field' => array(
							'title'			=> __('FedEx Document Field', 'ph_meta_data_to_shipping_labels'),
							'type'			=> 'select',
							'options'		=> array(
								'inv_field' 		=> __('INV Field - Shipping Label', 'ph_meta_data_to_shipping_labels'),
								'po_field' 			=> __('PO Field - Shipping Label', 'ph_meta_data_to_shipping_labels'),
								'dept_field' 		=> __('DEPT Field - Shipping Label', 'ph_meta_data_to_shipping_labels'),
								'consignee_tax' 	=> __('Consignee TIN - Commercial Invoice', 'ph_meta_data_to_shipping_labels'),
							),
							'description'	=> __('','ph_meta_data_to_shipping_labels'),
						),

						'ph_ups_label_enable' => array(
							'title'			=> __('Enable Meta for UPS', 'ph_meta_data_to_shipping_labels'),
							'type'			=> 'checkbox',
							'default' 		=> 'no',
							'label'			=> __('Enable','ph_meta_data_to_shipping_labels'),
							'description'	=> __('','ph_meta_data_to_shipping_labels'),
						),

						'ph_ups_label_field' => array(
							'title'			=> __('UPS Shipping Label Field', 'ph_meta_data_to_shipping_labels'),
							'type'			=> 'select',
							'options'		=> array(
								'ref_2' 		=> __('Reference Number Field', 'ph_meta_data_to_shipping_labels'),
								'context'       =>__('Customer Context', 'ph_meta_data_to_shipping_labels'),
							),
							'description'	=> __('','ph_meta_data_to_shipping_labels'),
						),
					);
				}

			}

		}
	}

	add_action( 'woocommerce_shipping_init', 'PH_meta_data_to_shipping_labels' );

	function add_meta_to_customer_reference_plugin( $methods ) {
		$methods['PH_meta_data_to_shipping_labels'] = 'PH_Meta_Data_To_Shipping_Labels';
		return $methods;
	}

	add_filter( 'woocommerce_shipping_methods', 'add_meta_to_customer_reference_plugin' );

	/* Create a Settings link */
	function ph_meta_data_to_shipping_labels_settings_link( $links ) 
	{
		$settings_link = '<a href="admin.php?page=wc-settings&tab=shipping&section=ph_meta_data_to_shipping_labels">Settings</a>';
		array_push( $links, $settings_link );
		return $links;
	}

	add_action( 'admin_enqueue_scripts', 'ph_meta_to_label_scripts');

	function ph_meta_to_label_scripts() {

		wp_enqueue_script( 'ph_meta_to_label', plugins_url( '/js/ph_common.js', __FILE__ ));
	}

	add_filter( "plugin_action_links_".plugin_basename(__FILE__), 'ph_meta_data_to_shipping_labels_settings_link');

	/** FedEx Plugin Code Start **/
	add_filter( 'wf_fedex_request', 'ph_update_meta_data_to_fedex_customer_references', 20, 4 );

	function ph_update_meta_data_to_fedex_customer_references( $request, $order, $parcel, $packages ) {

		$settings 		= get_option('woocommerce_ph_meta_data_to_shipping_labels_settings');
		$fedexSettings  = get_option('woocommerce_wf_fedex_woocommerce_shipping_settings');

		$metaType 		= ( isset( $settings['ph_meta_data_type'] ) && !empty( $settings['ph_meta_data_type'] ) ) ? $settings['ph_meta_data_type'] : 'order_meta';
		$metaKey 		= ( isset( $settings['ph_meta_data_key'] ) && !empty( $settings['ph_meta_data_key'] ) ) ? $settings['ph_meta_data_key'] : '';
		$metatext      = ( isset( $settings['ph_custom_field'] ) && !empty( $settings['ph_custom_field'] ) ) ? $settings['ph_custom_field'] :'[ORDER_ID]';
		$labelEnabled 	= ( isset( $settings['ph_fedex_label_enable'] ) && !empty( $settings['ph_fedex_label_enable'] ) ) ? $settings['ph_fedex_label_enable'] : 'no';
		$labelField 	= ( isset( $settings['ph_fedex_label_field'] ) && !empty( $settings['ph_fedex_label_field'] ) ) ? $settings['ph_fedex_label_field'] : 'inv_field';
		$dataType     = ( isset( $settings['ph_data_type'] ) && !empty( $settings['ph_data_type'] ) ) ? $settings['ph_data_type'] : 'metaData';

		$products 		= array();

		// Return if Meta Key is not Specified
		if( (empty($metaKey) && $dataType=='metaData') || $labelEnabled == 'no'){
			return $request;
		}

		$productsInRequest 	= isset($parcel['packed_products']) && !empty($parcel['packed_products']) ? $parcel['packed_products'] : array();

		// Return in case Parcel doesn't contain any Products
		if( empty($productsInRequest) ) {
			return $request;
		}

		foreach ($productsInRequest as $product) {
			
			$products[] = $product->get_id();
		}

		// Return in case Products array is empty
		if( empty($products) ) {
			return $request;
		}

		// Take Unique Products
		$products =  array_unique($products,1);

        if($dataType=='customData'){

		$metatext = str_replace( '[ORDER_ID]',$order->get_id(), $metatext );
		$metaValue=$metatext;

		} else {

		$metaValue 	= ph_get_meta_value( $metaType, $metaKey, $order, $products );

        }

		// Update Shipment Request if Meta Value is not empty
		if( !empty($metaValue) ) {

			if( isset($request['RequestedShipment']) && isset($request['RequestedShipment']['RequestedPackageLineItems']) && is_array($request['RequestedShipment']['RequestedPackageLineItems']) ) {

				$customerReferenceType = array(
					'inv_field' 	=> 'INVOICE_NUMBER',
					'po_field' 		=> 'P_O_NUMBER',
					'dept_field' 	=> 'DEPARTMENT_NUMBER',
				);
				
				if ( $labelField == 'consignee_tax' ) {

					$tintype 	= isset( $fedexSettings['tin_type'] ) ? $fedexSettings['tin_type'] : 'BUSINESS_STATE';

					$request['RequestedShipment']['Recipient']['Tins'] =  array(
						'TinType'	=> $tintype,
						'Number'	=> $metaValue,
					);

				} else {

					foreach ($request['RequestedShipment']['RequestedPackageLineItems'] as $key => $packageLineItems) {
						$existingReference=false;
						foreach ($packageLineItems['CustomerReferences'] as $key2 => $value) {

							if($packageLineItems['CustomerReferences'][$key2]['CustomerReferenceType']==$customerReferenceType[$labelField])
							{
								$existingReference=true;
								$packageLineItems['CustomerReferences'][$key2]['Value'] = $metaValue;
								$request['RequestedShipment']['RequestedPackageLineItems']=$packageLineItems;

							} 
						}
						if(!$existingReference){
							$request['RequestedShipment']['RequestedPackageLineItems'][$key]['CustomerReferences'][] = array(

								'CustomerReferenceType'		=> $customerReferenceType[$labelField],
								'Value'						=> $metaValue,
							);

						}
				
				}
			}

		}


		return $request;
	   }
	}
	/** FedEx Plugin Code End **/

	/** UPS Plugin Code Start **/
	add_filter( 'ph_ups_shipment_confirm_packages', 'ph_update_meta_data_to_ups_reference_number', 20, 3 );
	add_filter( 'ph_ups_update_customer_context_value', 'ph_update_meta_data_to_ups_context_number');


	function ph_update_meta_data_to_ups_context_number($id){
		$order=wc_get_order( $id );
		$product_in_array=array();
		$products=$order->get_items();
		foreach ( $products as $item ) {  
		    $value=$item->get_variation_id();
			if(empty($value))
			{
				$value=$item->get_product_id();
			}
			$product_in_array[] = $value; 
			
		}
		$settings 		= get_option('woocommerce_ph_meta_data_to_shipping_labels_settings');

		$customvalue      = ( isset( $settings['ph_custom_field'] ) && !empty( $settings['ph_custom_field'] ) ) ? $settings['ph_custom_field'] :'[ORDER_ID]';
		$labelEnabled 	= ( isset( $settings['ph_ups_label_enable'] ) && !empty( $settings['ph_ups_label_enable'] ) ) ? $settings['ph_ups_label_enable'] : 'no';
		$labelField 	= ( isset( $settings['ph_ups_label_field'] ) && !empty( $settings['ph_ups_label_field'] ) ) ? $settings['ph_ups_label_field'] : 'ref_2';
		$metaType 		= ( isset( $settings['ph_meta_data_type'] ) && !empty( $settings['ph_meta_data_type'] ) ) ? $settings['ph_meta_data_type'] : 'order_meta';
		$metaKey 		= ( isset( $settings['ph_meta_data_key'] ) && !empty( $settings['ph_meta_data_key'] ) ) ? $settings['ph_meta_data_key'] : '';
		$dataType     = ( isset( $settings['ph_data_type'] ) && !empty( $settings['ph_data_type'] ) ) ? $settings['ph_data_type'] : 'metaData';
        
		if( $labelEnabled == 'no' ) {
			return $id;
		}

		if($dataType=='customData' && $labelField=='context' ){

			$customvalue = str_replace('[ORDER_ID]',$id, $customvalue );

		}

		elseif(!empty($metaKey) && $dataType =='metaData' && $labelField=='context' ){
			
			$customvalue= ph_get_meta_value( $metaType, $metaKey, $order, $product_in_array );

		} else {

			$customvalue=$id;

		}

		    return $customvalue;
	}

	function ph_update_meta_data_to_ups_reference_number( $package, $package_with_items, $order_id ) {

		$productsInRequest 	= isset($package_with_items['Package']) && isset($package_with_items['Package']['items']) && !empty($package_with_items['Package']['items']) ? $package_with_items['Package']['items'] : array();

		// Return in case Parcel doesn't contain any Products
		if( empty($productsInRequest) ) {
			return $package;
		}

		$settings 		= get_option('woocommerce_ph_meta_data_to_shipping_labels_settings');
		$metaType 		= ( isset( $settings['ph_meta_data_type'] ) && !empty( $settings['ph_meta_data_type'] ) ) ? $settings['ph_meta_data_type'] : 'order_meta';
		$metaKey 		= ( isset( $settings['ph_meta_data_key'] ) && !empty( $settings['ph_meta_data_key'] ) ) ? $settings['ph_meta_data_key'] : '';
		$metatext      = ( isset( $settings['ph_custom_field'] ) && !empty( $settings['ph_custom_field'] ) ) ? $settings['ph_custom_field'] :'[ORDER_ID]';
		$labelEnabled 	= ( isset( $settings['ph_ups_label_enable'] ) && !empty( $settings['ph_ups_label_enable'] ) ) ? $settings['ph_ups_label_enable'] : 'no';
		$labelField 	= ( isset( $settings['ph_ups_label_field'] ) && !empty( $settings['ph_ups_label_field'] ) ) ? $settings['ph_ups_label_field'] : 'ref_2';
		$dataType     = ( isset( $settings['ph_data_type'] ) && !empty( $settings['ph_data_type'] ) ) ? $settings['ph_data_type'] : 'metaData';

		$products 		= array();

		// Return if Meta Key is not Specified
	    if((empty($metaKey) && $dataType=='metaData') ||$labelEnabled == 'no' || $labelField=='context') {
			return $package;
		}

		foreach ($productsInRequest as $product) {
			
			$products[] = $product->get_id();
		}

		// Return in case Products array is empty
		if( empty($products) ) {
			return $package;
		}

		// Take Unique Products
		$products =  array_unique($products,1);

		if($dataType=='customData'){

		$metatext = str_replace( '[ORDER_ID]',$order_id, $metatext );
		$metaValue=$metatext;
		
		} else {

		$metaValue 	= ph_get_meta_value( $metaType, $metaKey, wc_get_order( $order_id ), $products );
        
        }

		// Update Shipment Request if Meta Value is not empty
		if( !empty($metaValue) && isset($package['Package']) ) {

			$allReferences 	= array();
			$metaValue 		= ( strlen( $metaValue ) >= 35 ) ? substr( $metaValue, 0, 32 ).'...' : $metaValue;

			if( isset($package['Package']['ReferenceNumber']) ) {

				$allReferences[] 	= array(

					'ReferenceNumber'=> array(

						'Code'  => $package['Package']['ReferenceNumber']['Code'],
						'Value'	=> $package['Package']['ReferenceNumber']['Value'],
					)
				);

			} else {

				$allReferences[] 	= array(

					'ReferenceNumber'=> array(

						'Code'  => 'PO',
						'Value'	=> $order_id,
					)
				);
			}

			$allReferences[] 	= array(

				'ReferenceNumber'=> array(

					'Code'  => '02',
					'Value'	=> $metaValue,
				)
			);

			$package['Package']['ReferenceNumber']	=	array_merge( array('multi_node'=>1), $allReferences );
		}


		return $package;
	 }
	
	/** UPS Plugin Code End **/

	// Common Function to get Meta Value
	function ph_get_meta_value( $metaType, $metaKey, $order, $products ) {

		$metaValue		= '';

		if( $metaType == 'order_meta' ) {

			if( $order instanceof WC_Order ) {

				$orderId = $order->get_id();

				$metaValue = get_post_meta( $orderId, $metaKey, true );
			}

		} else {

			if( $order instanceof WC_Order ) {

				// Get Order Items
				$orderItems 	= $order->get_items();

				foreach ($orderItems as $item) {

					// Get Product Id
					$orderItemId = $item->get_variation_id();

					if( empty($orderItemId) ) {
						$orderItemId = $item->get_product_id();
					}

					// Check if Current Product is in the Requesting Package
					if(  in_array($orderItemId, $products) ) {

						if( $metaType == 'order_item_meta' ) {

							$value 		= $item->get_meta( $metaKey );

						} else if( $metaType == 'product_meta' ) {

							$value 		= get_post_meta( $orderItemId, $metaKey, true );
						}

						// Append the values in case, multiple products are there
						if( !empty($value) ) {
							$metaValue .= $value.', ';
						}
					}
				}

				// Remove the , appended to the end
				if( !empty($metaValue)  ) {
					$metaValue = rtrim($metaValue, ', ');
				}
			}
		}

		return $metaValue;
	}